
/**
 * 遍历映射配置，构建映射关系
 * @param dataMapping   映射配置
 * @param rawData   源数据：&nbsp;&nbsp;&nbsp;&nbsp;若有此输入，将认为当前是更新动作、&nbsp;&nbsp;无此输入则认为当前是插入动作
 */
function mapping(dataMapping, rawData) {
    window.dataSet = {};
    if(!rawData) {
        window.dataSet.isInsert = true;
        var rawData = {};
    }else {
        window.dataSet.isInsert = false;
    }
    let mapper = {};
    delete dataMapping.Mapper;
    let entitySet =  Object.entries(dataMapping);
    let returnData = {};
    tryForEach(entitySet,(entity) => {
        let beanName = entity[0];
        if(!isExist(rawData, beanName)) { throw {}; }
        let entityData =  rawData[beanName];
        let propertyNames = entity[1];
        tryForEach(propertyNames,(item) => {
            if(typeof item === "string") {
                if(!isExist(entityData, item, beanName)) { throw {}; }
                associate(entityData, item, item, beanName,mapper, returnData);
                return;
            } else if(item instanceof Array) {
                // Grid需要单独赋值,这里不设置returnData. 页面Input有足够的映射信息,这里只记录 property: objName
                tryForEach(item, (property) => {
                    mapper[property] = beanName;
                });
                return;
            } else if(item instanceof Object) {
                let entry = Object.entries(item)[0];
                let property = entry[0];
                if(!isExist(entityData, property, beanName)) { throw {}; }
                let alias = entry[1];
                associate(entityData, property, alias, beanName,mapper, returnData)
            } else {
                console.error('The configuration of "'+item+'" in "'+beanName+'" is not recognized,  and the required,' +
                    ' parameters are ( "propertyName" or "{ propertyName: alias }" ).');
                throw {};
            }
        });
    });
    dataMapping.Mapper = mapper;
    // ---------------- 暂不处理无法执行到此处的情况 ----------------
    // 这里写死 映射对象为：‘dataMapping’
    window.dataMapping = dataMapping;
    // 这里写死 数据集：‘dataSet: { rawData: {}, newData: {} }’
    window.dataSet.rawData = rawData;
    window.dataSet.newData = JSON.parse(JSON.stringify(dataSet.rawData));
    return returnData;
}

/**
 *  映射关系 => elementName(alias): { objName: property }
 * @param data  数据源
 * @param property  属性
 * @param alias 别名
 * @param objName   对象名
 * @param mapper    返回值: 映射关系
 * @param returnData    返回值: 页面显示数据集
 */
function associate(data, property, alias, objName, mapper, returnData) {
    mapper[alias] = {};
    mapper[alias][objName] = property;
    returnData[alias] = data[property];
}

function testTryForEach() {
    // 直接修改原型属性， 会导致当前UI grid 组件 input 不可用，未知原因。
    Array.prototype.tryForEach = function (callback) {
        try {
            this.forEach(callback);
        } catch { }
    }
}

var tryForEach = function(arrayObj, callback) {
    try {
        arrayObj.forEach(callback);
    } catch { }
}

/**
 * 检测数据中是否存在对应属性
 * @param obj   需要检测数据
 * @param property  需要检测的属性
 * @param objName   检测的对象名
 * @returns {boolean}
 */
function isExist(obj, property, objName) {
    if(window.dataSet.isInsert) {
        obj[property] = {};
        return true;
    } else {
        if(!objName) { objName = 'rawData'; }
        // 这里注意，后端返回的类名是否与映射配置一致
        if(!obj[property]) {
            console.error('No data for name of ' + property + ' in ' + objName + ' !');
            return false;
        }
    }
    return true;
}

// 监听 input 管理页面显示的 Data，提交数据时，只提交修改过的 Data
function inputEventListener() {
    const inputElements = document.querySelectorAll('input');
    inputElements.forEach(item => {
        item.addEventListener('input', (event) => {
            let mapper = window.dataMapping.Mapper;
            let property = event.target.getAttribute('name');
            let beanMapping = mapper[property];
            if(!beanMapping) {
                console.error("The configuration was not found in the 'dataMapping' : '"+property+"'")
            }
            if(typeof beanMapping === "string") {
                debugger;
            }
            if(beanMapping instanceof Object) {
                mappings = Object.entries(beanMapping);
                tryForEach(mappings, (entry) => {
                    let beanName = entry[0];
                    let property = entry[1];
                    let newData = window.dataSet.newData;
                    newData[beanName][property] = event.target.value;
                    console.log('Input value changed:', event.target.value);
                })
            }

        });
    });
}

function checkUpdata() {
    let rawData = window.dataSet.rawData;
    let newData = window.dataSet.newData;
    window.dataSet.upData = {};
    let rawEntry = Object.entries(rawData);
    rawEntry.forEach(raw => {
        let beanName = raw[0];
        let rawHash = getHashCode(JSON.stringify(rawData[beanName]));
        let newHash = getHashCode(JSON.stringify(newData[beanName]));
        if(!(rawHash === newHash)) {
            window.dataSet.upData[beanName] = newData[beanName];
        }
    })
}

/**
 * 用于判断字符串是否更改
 * @param str
 * @param caseSensitive 是否区分大小写 Default: true
 * @returns {number}
 */
function getHashCode (str, caseSensitive) {
    if(caseSensitive === undefined) { caseSensitive = true;}
    if(!caseSensitive) {
        str = str.toLowerCase();
    }
    if (str.length === 0) return 0;
    var len = str.length;
    var hash = 1315423911, i, ch;
    for (i = len - 1; i >= 0; i--) {
        ch = str.charCodeAt(i);
        hash = (hash << 5) + ch + (hash >> 2);
    }
    var hash2 = hash ^ (hash << 7) ^ (hash >> 6);
    var hash3 = hash2 ^ (hash2 << 1) ^ (hash2 >> 13);
    if (len < 50) {
        var hash4 = this.getHashCode(str + str + str, caseSensitive);
        return (hash ^ hash2 ^ hash3  ^ hash4) & 0x7FFFFFFF;
    } else {
        return (hash ^ hash2 ^ hash3 >> 3 ) & 0x7FFFFFFF;
    }
}

function runObserver() {
    observer.observe(document.querySelector('#panelContainer'), mutationConfig);
}

// 补充方案 监听 Value 属性
const observer = new MutationObserver((mutations) => {
    mutations.forEach((mutation) => {
        if(mutation.type === 'attributes'&& mutation.attributeName === 'value') {
            console.log('属性变化节点 ： ' + mutation.target);
            console.log('变化属性名 ： ' + mutation.attributeName);
            console.log('变化属性值 ： ' + mutation.target.value);
        }
    })
})
const mutationConfig = { attributes: true, childList: true, subtree: true}